from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class LacreiidConfig(AppConfig):
    default_auto_field = "django.db.models.BigAutoField"
    name = "lacreiid"
    label = "lacreiid"
    verbose_name = _("Lacrei ID")
