from django.utils.translation import gettext as _

PROFESSIONAL_ALREADY_APPROVED_FOR_STEP = _(
    "Pessoa profissional já foi approvada para esta etapa"
)

BOARD_VERIFICATION_NOT_APPROVED_YET = _(
    "Inclua uma validação do número conselho antes de prosseguir com esta etapa."
)


PROFESSIONAL_VALIDATION_PENDING_ADMIN = _("Pendente Lacrei")
PROFESSIONAL_VALIDATION_PENDING_DATA_INPUT = _("Pendente Profissional")
